<?php

namespace uAfrica_Shipping\app;

/**
 * Class Shipping
 *
 * @package uAfrica_Shipping\app
 */
class Shipping extends \WC_Shipping_Method {

	/**
	 * Constructor for your shipping class
	 *
	 * @access public
	 * @return void
	 */
	public function __construct() {
		// phpcs:ignore WordPress.WP.I18n.NoHtmlWrappedStrings
		$description              = __( '<div>When <strong>uAfrica Shipping Rates</strong> are enabled, your buyers will receive the rates that you have configured on uAfrica Shipping Rates. For this to work you need an active <a href="https://www.uafrica.com" target="_new">uAfrica Shipping account</a></div>', 'uafrica-shipping' );
		$this->id                 = 'uafrica_shipping';
		$this->title              = __( 'uAfrica shipping', 'uafrica-shipping' );
		$this->method_title       = __( 'uAfrica Shipping Rates', 'uafrica-shipping' );
		$this->method_description = $description;
		$this->enabled            = 'yes'; // This can be added as an setting but for this example its forced enabled.
		$this->init();
	}

	/**
	 * Init your settings
	 *
	 * @access public
	 * @return void
	 */
	public function init() {
		// Load the settings API.
		$this->init_form_fields(); // This is part of the settings API. Override the method to add your own settings.
		$this->init_settings(); // This is part of the settings API. Loads settings you previously init.
		// Save settings in admin if you have any defined.
		add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
	}

	/**
	 * Init the fields for the admin.
	 */
	public function init_form_fields() {
		$this->form_fields = array(
			'enabled'       => array(
				'title'       => __( 'Enable', 'uafrica-shipping' ),
				'type'        => 'checkbox',
				'description' => __(
					'When this setting is enabled, your customers will be
				presented with shipping rates at checkout, as configured on the uAfrica
				platform under Shipping Rates.',
					'uafrica-shipping'
				),
				'default'     => 'no',
			),
			'hide'          => array(
				'title'       => __( 'Hide WooCommerce Shipping Rates', 'uafrica-shipping' ),
				'type'        => 'checkbox',
				'description' => __( 'Hide other WooCommerce shipping rates if uAfrica Shipping return rates.', 'uafrica-shipping' ),
				'default'     => 'yes',
			),
			'Test_API_Call' => array(
				'title'       => __( 'Makes a test call to the uAfrica-shipping API', 'uafrica-shipping' ),
				'type'        => 'custom_display',
				'description' => __( 'Makes a test call to the uAfrica-shipping API to ensure that the connection is set up properly.', 'uafrica-shipping' ),
				'value'       => 'Test API',
			),
		);
	}

	/**
	 * Generates an HTML template for displaying our custom API test call result
	 * that can be called through type custom_display
	 */
	public function generate_custom_display_html() {
		?>
		<div class="wrap">
			<div class="cmp-flex-tabs__content" style="">
				<?php $this->test_shipping(); ?>
			</div>
		</div>
		<?php
	}

	/**
	 * Calculate the shipping costs of the uAfrica shipping.
	 * Might return multiple couriers with different rates.
	 *
	 * @param array $package The shopping cart details.
	 */
	public function calculate_shipping( $package = array() ) {
		$uafrica_shipping_settings = get_option( 'woocommerce_uafrica_shipping_settings' );
		// Is uAfrica shipping enabled?
		if ( empty( $uafrica_shipping_settings['enabled'] ) || 'no' === $uafrica_shipping_settings['enabled'] ) {
			return;
		}

		$url = str_replace( 'DOMAIN', Admin::get_api_domain(), UAFRICA_SHIPPING_API_SHIPPING_METHODS );
		$r   = wp_remote_post(
			$url,
			array(
				'body'        => $this->get_api_formatted_body( $package ),
				'redirection' => 0,
				'timeout'     => 12, // 12 seconds before timeout
			)
		);

		if ( wp_remote_retrieve_response_code( $r ) < 200 || wp_remote_retrieve_response_code( $r ) > 299 ) {
			return; // Invalid status code, don't bother trying.
		}
		$rates = json_decode( wp_remote_retrieve_body( $r ) );
		if ( empty( $rates->rates ) ) {
			return; // There are no rates in the body.
		}
		$i = 1;
		foreach ( $rates->rates as $rate ) {
			$rate = array(
				'id'    => "{$this->id}:{$i}",
				'label' => $rate->service_name,
				'cost'  => (string) ( $rate->total_price / 100 ),
			);
			$this->add_rate( $rate );
			$i ++;
		}
	}

	/**
	 * Format package data into a valid format for API call.
	 *
	 * @param array $package The shopping cart details.
	 *
	 * @return array[]
	 */
	protected function get_api_formatted_body( array $package ): array {
		$formatted_data = array(
			'rate' => array(
				'origin'      => $this->format_origin( $package ),
				'destination' => $this->format_destination( $package ),
				'currency'    => 'ZAR',
				'locale'      => substr( get_locale(), 0, 2 ),
			),
		);

		// Add products to the cart.
		if ( ! empty( $package['contents'] ) ) {
			$formatted_data['rate']['items'] = $this->format_cart_items( $package['contents'] );
		}

		return $formatted_data;
	}

	/**
	 * Format the destination for the API.
	 *
	 * @param array $package raw cart data.
	 *
	 * @return array
	 */
	protected function format_destination( array $package ): array {
		$formatted_destination = array(
			'country'      => $package['destination']['country'],
			'postal_code'  => $package['destination']['postcode'],
			'province'     => '',
			'city'         => $package['destination']['city'],
			'name'         => null,
			'address1'     => $package['destination']['address_1'],
			'address2'     => $package['destination']['address_2'],
			'address3'     => null,
			'phone'        => '',
			'fax'          => '',
			'email'        => null,
			'address_type' => null,
			'company_name' => '',
		);

		// Provinces.
		if ( ! empty( $package['destination']['state'] ) ) {
			$formatted_destination['province'] = $package['destination']['state'];
		}

		return $formatted_destination;
	}

	/**
	 * Format the origin for the API.
	 * The origin is where the store is located.
	 *
	 * @return array
	 */
	protected function format_origin(): array {
		$store_address   = get_option( 'woocommerce_store_address' );
		$store_address_2 = get_option( 'woocommerce_store_address_2' );
		$store_city      = get_option( 'woocommerce_store_city' );
		$store_postcode  = get_option( 'woocommerce_store_postcode' );

		// The country/state.
		$store_raw_country = get_option( 'woocommerce_default_country' );

		// Split the country/state.
		$split_country = explode( ':', $store_raw_country );

		$store_country  = ( ! empty( $split_country[0] ) ) ? $split_country[0] : '';
		$store_province = ( ! empty( $split_country[1] ) ) ? $split_country[1] : '';

		$formatted_origin = array(
			'country'      => $store_country,
			'postal_code'  => $store_postcode,
			'province'     => $store_province,
			'city'         => $store_city,
			'name'         => null,
			'address1'     => $store_address,
			'address2'     => $store_address_2,
			'address3'     => null,
			'phone'        => '',
			'fax'          => '',
			'email'        => null,
			'address_type' => null,
			'company_name' => '',
		);

		return $formatted_origin;
	}

	/**
	 * Format the items in the cart for the API.
	 *
	 * @param array $cart_items Raw cart items.
	 *
	 * @return array
	 */
	protected function format_cart_items( array $cart_items ): array {
		$formatted_cart_items = array();
		foreach ( $cart_items as $item ) {
			/**
			 * Holds the product.
			 *
			 * @var \WC_Product $wc_product
			 */
			$wc_product = $item['data'];

			$requires_shipping = ( $wc_product->needs_shipping() ) ? '1' : '0';
			$taxable           = ( $wc_product->is_taxable() ) ? '1' : '0';

			$formatted_item = array(
				'name'                => $wc_product->get_name(),
				'sku'                 => $wc_product->get_sku(),
				'quantity'            => $item['quantity'],
				'price'               => ( ( (int) $wc_product->get_price() ) * 100 ), // convert to cents.
				'vendor'              => '',
				'requires_shipping'   => $requires_shipping,
				'taxable'             => $taxable,
				'fulfillment_service' => 'manual',
				'properties'          => null,
				'product_id'          => $wc_product->get_id(),
				'grams'               => wc_get_weight( $wc_product->get_weight(), 'g' ),
			);

			// If it has a parent it's a variation or configurable.
			if ( $wc_product->get_parent_id() ) {
				$formatted_item['product_id'] = $wc_product->get_parent_id();
				$formatted_item['variant_id'] = $wc_product->get_id();
			}

			$formatted_cart_items[] = $formatted_item;
		}

		return $formatted_cart_items;
	}

	/**
	 * Run a test call to uAfrica shipping Rates API to see if the API can be reached and give a
	 * usefull error if it can't
	 */
	public function test_shipping() {
		$uafrica_shipping_settings = get_option( 'woocommerce_uafrica_shipping_settings' );
		// Is uAfrica shipping enabled?
		if ( empty( $uafrica_shipping_settings['enabled'] ) || 'no' === $uafrica_shipping_settings['enabled'] ) {
			return;
		}
		// Create API URL.
		$url = str_replace( 'DOMAIN', Admin::get_api_domain(), UAFRICA_SHIPPING_API_SHIPPING_METHODS );
		// Call API using URL and Test Data.
		$r = wp_remote_post(
			$url,
			array(
				'body'        => $this->format_test_package(),
				'redirection' => 0,
				'timeout'     => 12, // 12 seconds before timeout
			)
		);
		// If a general PHP or WordPress error is returned when connecting display it.
		if ( is_wp_error( $r ) ) {
			echo "<div style='color: red; margin-top:10px'> ";
			echo 'Failed to connect uAfrica Rates API : ';
			echo $r->get_error_message(); //phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			echo '</div>';
		} else {
			// Inspect the returned API data to see if rates are returned.
			$returnbody = json_decode( $r['body'], true );
			// Displays a uAfrica specific error if it has been returned from the API.
			if ( array_key_exists( 'success', $returnbody ) && false === $returnbody['success'] ) {
				if ( 'Not Found' === $returnbody['data']['message'] ) {
					echo "<div style='color: red; margin-top:10px'> ";
					echo 'Your WooCommerce channel is currently not enabled on uAfrica. Please visit ';
					echo '<a href="https://www.uafrica.com" target="_new">uAfrica.com</a>';
					echo ' and enable your WooCommerce channel to be able to receive rates';
					echo '</div>';
				} else {
					echo "<div style='color: red; margin-top:10px'> ";
					echo $returnbody['data']['message'];  //phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					echo '</div>';
				}
			} elseif ( array_key_exists( 'total_price', $returnbody['rates'][0] ) ) {
				// Return a successful response if the call was successful and rates are returned.
				echo "<div style='color: green; margin-top:10px'> ";
				echo 'uAfrica Rates API connected';
				echo '</div>';
			} else {
				// if rates are not returned give corresponding error.
				echo "<div style='color: red; margin-top:10px'> ";
				echo 'uAfrica rates is currently not enabled on uAfrica. Please visit ';
				echo '<a href="https://www.uafrica.com" target="_new">uAfrica.com</a>';
				echo ' and enable your WooCommerce channel to be able to receive rates';
				echo '</div>';
			}
		}
	}

	/**
	 * Format test package data into a valid format for API call.
	 *
	 * @return array[]
	 */
	protected function format_test_package() {
		$test_data = array(
			'rate' => array(
				'origin'      => array(
					'country'      => 'ZA',
					'postal_code'  => '0081',
					'province'     => 'GT',
					'city'         => 'Pretoria',
					'name'         => null,
					'address1'     => '36 Marelu Street',
					'address2'     => 'Six Fountains Estate',
					'address3'     => null,
					'phone'        => '0836574497',
					'fax'          => '',
					'email'        => null,
					'address_type' => null,
					'company_name' => 'Jamie Ds Emporium',
				),
				'destination' => array(
					'country'      => 'ZA',
					'postal_code'  => '0081',
					'province'     => 'GT',
					'city'         => 'Pretoria',
					'name'         => null,
					'address1'     => '36 Marelu Street',
					'address2'     => 'Six Fountains Estate',
					'address3'     => null,
					'phone'        => '0836574497',
					'fax'          => '',
					'email'        => null,
					'address_type' => null,
					'company_name' => 'Jamie Ds Emporium',
				),
				'items'       => array(
					'0' => array(
						'name'                => 'Short Sleeve T - Shirt',
						'sku'                 => null,
						'quantity'            => '1',
						'price'               => '19990',
						'vendor'              => 'Jamie Ds Emporium',
						'requires_shipping'   => '1',
						'taxable'             => '1',
						'fulfillment_service' => 'manual',
						'properties'          => null,
						'product_id'          => '48447225880',
						'variant_id'          => '258644705304',
					),
				),

				'currency'    => 'ZAR',
				'locale'      => 'en',
			),
		);

		return $test_data;
	}
}
