<?php

namespace uAfrica_Shipping\app;

class WooCommerce {

	/**
	 * Save the total weight of the items in the order as part of order meta so that it is included in order webhooks.
	 *
	 * @param int $order_id ID of the order.
	 * @param array|\WC_Order|null Posted order data, or the existing order.
	 *
	 * @return void
	 */
	public static function include_total_grams_in_order( $order_id, $order = null ) {
		if ( ! function_exists( 'WC' ) ) {
			return;
		}

		$weight = 0;
		$cart   = WC()->cart;
		if ( $cart !== null ) {
			// Grab total weight from cart.
			$weight = $cart->get_cart_contents_weight();
		} elseif ( $order instanceof \WC_Order ) {
			// Order was likely created/edited on admin or via REST API. Determine total weight from product data.
			foreach ( $order->get_items() as $item ) {
				$product = WC()->product_factory->get_product( $item->get_data()['product_id'] );
				if ( $product === false ) {
					continue;
				}

				$weight += (float) $product->get_weight() * $item->get_quantity();
			}
		}

		$weight = wc_get_weight( $weight, 'g' );
		update_post_meta( $order_id, 'uafrica_total_grams', $weight );
	}

	/**
	 * Determine whether or not the suburb_at_checkout setting is active.
	 *
	 * @return bool
	 */
	private static function has_suburb_at_checkout(): bool {
		$options            = get_option( 'uafrica' );
		$suburb_at_checkout = $options['suburb_at_checkout'] ?? 1;

		return (bool) $suburb_at_checkout;
	}

	/**
	 * Add a suburb field to admin address fields so that it can be edited per order.
	 *
	 * @param array $fields Admin address edit fields.
	 *
	 * @return array
	 */
	public static function add_suburb_to_admin_address_fields( $fields ) {
		if ( ! is_array( $fields ) || empty( $fields ) ) {
			return [];
		}

		if ( ! self::has_suburb_at_checkout() ) {
			return $fields;
		}

		$out = [];
		foreach ( $fields as $key => $field ) {
			if ( $key === 'city' ) {
				$out['suburb'] = [
					'label' => 'Suburb',
					'show'  => true,
					'style' => '',
				];
			}
			$out[ $key ] = $field;
		}

		return $out;
	}

	/**
	 * Add suburb fields to checkout.
	 *
	 * @param array $fields Checkout fields.
	 *
	 * @return array
	 */
	public static function add_suburb_to_checkout_fields( $fields ) {
		if ( ! is_array( $fields ) || empty( $fields ) || ! function_exists( 'WC' ) ) {
			return [];
		}

		if ( ! self::has_suburb_at_checkout() ) {
			return $fields;
		}

		$cart = WC()->cart;
		if ( $cart && ! $cart->needs_shipping() ) {
			return $fields;
		}

		$suburb = [
			'label'    => 'Suburb',
			'class'    => [ 'form-row-wide', 'address_field' ],
			'priority' => 65,
		];

		$fields['shipping']['shipping_suburb'] = $suburb;
		$fields['billing']['billing_suburb']   = $suburb;

		return $fields;
	}
}
